#!/bin/bash

PATH='/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin'
BIN_SQLITE3="$(which sqlite3)"
BIN_IPSET="$(which  ipset)"
BIN_IPTABLES="$(which iptables)"

ALLOW_LAN_IN_TCP=''
ALLOW_LAN_IN_UDP='123,53'
DROP_IN_TCP='53,111,389,636,2812,4369,5038,5280,6789'
DROP_IN_UDP='53,111,6050'
LOCAL_NETWORKS='10.0.0.0/8,100.64.0.0/10,172.16.0.0/12,192.168.0.0/16'
CUSTOM_SOURCE="
  /etc/default/iptables
"
RTP_CONFIG='/rw2/etc/callweaver/rtp-wms.conf'


__LOG() {

  local name message xargs

  name="${0##*/}"
  message="${2}"
  xargs="${3}"

  case "${1}" in
    crit)   severity='crit'    ;;
    err)    severity='err'     ;;
    warn)   severity='warning' ;;
    notice) severity='notice'  ;;
    debug)  severity='debug'   ;;
    *)      severity='info'    ;;
  esac

  if [ "${message}" ]; then
    echo ${xargs} "${message}" | tee >(systemd-cat -t "${name}" -p "${severity}")
  else
    systemd-cat -t "${name}" -p "${severity}"
  fi

}


__IPSET_ALLOC() {

  local table table_list settings

  table="${1:-ipset-test}"
  settings="${2:-hash:net}"
  table_list=($(${BIN_IPSET} list -n))

  [[ ! " ${table_list[*]} " =~ " ${table} " ]] && ${BIN_IPSET} create ${table} ${settings}

}


__IPSET_SWAP() {

  local table tmp_table table_list

  table="${1:-ipset-test}"
  tmp_table="${2:-ipset-test_tmp}"
  table_list=($(${BIN_IPSET} list -n))

  if [[ " ${table_list[*]} " =~ " ${table} " ]] && [[ " ${table_list[*]} " =~ " ${tmp_table} " ]]; then
    ${BIN_IPSET} -W "${tmp_table}" "${table}"
    ${BIN_IPSET} destroy "${tmp_table}"
  else
    __LOG "err" "ERROR($FUNCNAME): some tables do not exist." 1>&2
    return 1
  fi

}


__SQLITE() {

  local db query

  query="${1:-.tables}"
  db="${2:-/rw2/var/kamailio/htables.db}"

  if [ -f "${db}" ]; then
    ${BIN_SQLITE3} "${db}" "${query}"
  else
    __LOG "err" "ERROR($FUNCNAME): db: [${db}] does not exist." 1>&2
    return 1
  fi

}


__INIT_WHITELIST(){

  local sqlite_table ipset_table

  sqlite_table='trusted_ips'
  ipset_table='WHITELIST_IP'

  # check for availability of all binaries
  if [ -z "${BIN_SQLITE3}" ] || [ -z "${BIN_IPSET}" ] || [ -z "${BIN_IPTABLES}" ]; then
    __LOG "err" \
    "ERROR($FUNCNAME): some binaries is empty: sqlite3:[${BIN_SQLITE3}], ipset:[${BIN_IPSET}], iptables:[${BIN_IPTABLES}]." 1>&2
    return 1
  fi

  __IPSET_ALLOC "${ipset_table}"

  if [ -z "$(${BIN_IPTABLES} -vnL INPUT --line-numbers | grep -wo "match-set ${ipset_table}")" ]; then
      ${BIN_IPTABLES} -I INPUT 4 ! -i lo -m set --match-set ${ipset_table} src -j ACCEPT
  fi

  # check if the trusted_ips table exists in the sqlite database
  if [ "$(__SQLITE "SELECT name FROM sqlite_master WHERE name='${sqlite_table}';")" == "${sqlite_table}" ]; then

    __IPSET_ALLOC "${ipset_table}_TMP"

    # add all IPs to the ipset table
    for ip in $(__SQLITE "SELECT key_name FROM ${sqlite_table};" | grep -Eo "([0-9]{1,3}[\.]){3}[0-9]{1,3}"); do
      ${BIN_IPSET} add "${ipset_table}_TMP" "${ip}" 2>/dev/null
    done

    __IPSET_SWAP "${ipset_table}" "${ipset_table}_TMP"

  else
    __LOG "err" "ERROR($FUNCNAME): ${sqlite_table} table does not exist." 1>&2
  fi


}


# check for availability of all binaries
if [ -z "${BIN_IPTABLES}" ]; then
  __LOG "err" "ERROR: some binaries is empty: iptables:[${BIN_IPTABLES}], exit." 1>&2
  exit 1
fi


case "${1}" in
  # init rules for whitelist
  init_whitelist) __INIT_WHITELIST ;;
  # init main rules for iptables
  *)
    ## get the sip-rtp port for scan from a portal
    [ -f "${RTP_CONFIG}" ] && source ${RTP_CONFIG} || __LOG "warn" "WARNING: config not available: ${RTP_CONFIG}"
    [ "${rtpstart}" ] && RTP_PORT=$((rtpstart-1)) || RTP_PORT='10001'


    #----------------------
    ## cleanup rules
    #----------------------
    ${BIN_IPTABLES} -F
    ${BIN_IPTABLES} -X
    ${BIN_IPTABLES} -F -t nat
    ${BIN_IPTABLES} -X -t nat


    #----------------------
    ## ALLOWLAN filter
    #----------------------
    ${BIN_IPTABLES} -N ALLOWLAN
    ${BIN_IPTABLES} -A ALLOWLAN ! -i lo --src ${LOCAL_NETWORKS} -j ACCEPT


    #----------------------
    ## LOGDROP filter
    #----------------------
    ${BIN_IPTABLES} -N LOGDROP
    ${BIN_IPTABLES} -A LOGDROP -m limit --limit 10/min -j LOG --log-level debug --log-prefix "IPTABLES DROP:"
    ${BIN_IPTABLES} -A LOGDROP -j DROP


    #----------------------
    ## INVALID filter
    #----------------------
    ${BIN_IPTABLES} -N INVALID
    ${BIN_IPTABLES} -A INVALID -m limit --limit 10/min -j LOG --log-level debug --log-prefix "IPTABLES DROP (INVALID):"
    ${BIN_IPTABLES} -A INVALID -j DROP


    #----------------------
    ## TCPREJECT filter
    #----------------------
    ${BIN_IPTABLES} -N TCPREJECT
    ${BIN_IPTABLES} -A TCPREJECT -m limit --limit 10/min -j LOG --log-level debug --log-prefix "IPTABLES REJECT:"
    ${BIN_IPTABLES} -A TCPREJECT -p tcp -j REJECT --reject-with tcp-reset


    #----------------------
    ## INPUT filter
    #----------------------
    # allow from interface localhost, remote support
    ${BIN_IPTABLES} -A INPUT -i lo -j ACCEPT
    ${BIN_IPTABLES} -A INPUT -i tun0 -j ACCEPT

    # allow ports from local networks
    [ "${ALLOW_LAN_IN_TCP}" ] && ${BIN_IPTABLES} -A INPUT ! -i lo -p tcp -m multiport --dport ${ALLOW_LAN_IN_TCP} -j ALLOWLAN
    [ "${ALLOW_LAN_IN_UDP}" ] && ${BIN_IPTABLES} -A INPUT ! -i lo -p udp -m multiport --dport ${ALLOW_LAN_IN_UDP} -j ALLOWLAN

    # drop specific ports
    [ "${DROP_IN_TCP}" ] && ${BIN_IPTABLES} -A INPUT ! -i lo -p tcp -m multiport --dport ${DROP_IN_TCP} -j TCPREJECT
    [ "${DROP_IN_UDP}" ] && ${BIN_IPTABLES} -A INPUT ! -i lo -p udp -m multiport --dport ${DROP_IN_UDP} -j LOGDROP

    # accept related,established connections
    ${BIN_IPTABLES} -A INPUT ! -i lo -m conntrack --ctstate RELATED,ESTABLISHED -j ACCEPT

    ## dos protection for ssh
    ${BIN_IPTABLES} -A INPUT ! -i lo -p tcp -m multiport --dport 22,2222 -m conntrack --ctstate NEW -m recent --set --name SSH --rsource
    ${BIN_IPTABLES} -A INPUT ! -i lo -p tcp -m multiport --dport 22,2222 -m recent --update --seconds  60 --hitcount  3 --name SSH --rsource -j LOGDROP
    ${BIN_IPTABLES} -A INPUT ! -i lo -p tcp -m multiport --dport 22,2222 -m recent --update --seconds 300 --hitcount 15 --name SSH --rsource -j LOGDROP
    ## dos protection for port http\https
    ${BIN_IPTABLES} -A INPUT ! -i lo -p tcp --dport  80 -m conntrack --ctstate NEW -m recent --set --name HTTP --rsource
    ${BIN_IPTABLES} -A INPUT ! -i lo -p tcp --dport  80 -m recent --update --seconds  1 --hitcount 150 --name HTTP --rsource -j LOGDROP
    ${BIN_IPTABLES} -A INPUT ! -i lo -p tcp --dport  80 -m recent --update --seconds 10 --hitcount 200 --name HTTP --rsource -j LOGDROP
    ${BIN_IPTABLES} -A INPUT ! -i lo -p tcp --dport  80 -m recent --update --seconds 30 --hitcount 255 --name HTTP --rsource -j LOGDROP
    ${BIN_IPTABLES} -A INPUT ! -i lo -p tcp --dport 443 -m conntrack --ctstate NEW -m recent --set --name HTTPS --rsource
    ${BIN_IPTABLES} -A INPUT ! -i lo -p tcp --dport 443 -m recent --update --seconds  1 --hitcount 150 --name HTTPS --rsource -j LOGDROP
    ${BIN_IPTABLES} -A INPUT ! -i lo -p tcp --dport 443 -m recent --update --seconds 10 --hitcount 200 --name HTTPS --rsource -j LOGDROP
    ${BIN_IPTABLES} -A INPUT ! -i lo -p tcp --dport 443 -m recent --update --seconds 30 --hitcount 255 --name HTTPS --rsource -j LOGDROP
    ### dos protection for NTP
    ${BIN_IPTABLES} -A INPUT ! -i lo -p udp --dport 123 -m recent --set --name NTP --rsource
    ${BIN_IPTABLES} -A INPUT ! -i lo -p udp --dport 123 -m recent --update --seconds  1 --hitcount 100 --name NTP --rsource -j LOGDROP
    ${BIN_IPTABLES} -A INPUT ! -i lo -p udp --dport 123 -m recent --update --seconds 10 --hitcount 150 --name NTP --rsource -j LOGDROP
    ${BIN_IPTABLES} -A INPUT ! -i lo -p udp --dport 123 -m recent --update --seconds 30 --hitcount 250 --name NTP --rsource -j LOGDROP


    #----------------------
    ## FORWARD filter
    #----------------------
    # drop invalid pakages
    ${BIN_IPTABLES} -A FORWARD ! -i lo -m state --state INVALID -j INVALID


    #----------------------
    ## PREROUTING nat
    #----------------------
    ${BIN_IPTABLES} -t nat -N UDP-SIP
    ${BIN_IPTABLES} -t nat -A UDP-SIP -p udp --dport ${RTP_PORT} -j REDIRECT --to-port 5060
    ${BIN_IPTABLES} -t nat -A UDP-SIP -j RETURN
    ${BIN_IPTABLES} -t nat -A PREROUTING ! -i lo -p udp -j UDP-SIP


    ## Load custom settings
    for file in ${CUSTOM_SOURCE}; do
      [ -f "${file}" ] && source ${file} || __LOG "warn" "WARNING: custom source not available: ${file}"
    done

    ## init a whitelist
    __INIT_WHITELIST
  ;;
esac
