#!/bin/bash

AMAZON_API='169.254.169.254'
IPSET_TABLE='BLOCKED_IP'
BIN_IPSET="$(which ipset)"
BIN_IPTABLES="$(which iptables)"
BIN_IP="$(which ip)"
BIN_IFCONFIG="$(which ifconfig)"
BIN_USERMOD="$(which usermod)"
BIN_SYSTEMCTL="$(which systemctl)"


__LOG() {

	local name message xargs

	name="${0##*/}"
	message="${2}"
	xargs="${3}"

	case "${1}" in
		crit)   severity='crit'    ;;
		err)    severity='err'     ;;
		warn)   severity='warning' ;;
		notice) severity='notice'  ;;
		debug)  severity='debug'   ;;
		*)      severity='info'    ;;
	esac

	if [ "${message}" ]; then
		echo ${xargs} "${message}" | tee >(systemd-cat -t "${name}" -p "${severity}")
	else
		systemd-cat -t "${name}" -p "${severity}"
	fi

}


__CHECK_IPTABLES_RULE(){

	if [ -z "$(${BIN_IPTABLES} -vnL INPUT --line-numbers | grep -w '^1 ' | grep -w "${IPSET_TABLE}")" ]; then
		${BIN_IPTABLES} -I INPUT 1 ! -i lo -m set --match-set ${IPSET_TABLE} src -j DROP
	fi

}


__IPSET_ALLOC() {

	local table table_list settings

	table="${1:-ipset-test}"
	settings="${2:-hash:net}"
	table_list=($(${BIN_IPSET} list -n))

	[[ ! " ${table_list[*]} " =~ " ${table} " ]] && ${BIN_IPSET} create ${table} ${settings}

}


__IPSET_SWAP() {

	local table table_tmp table_list

	table="${1:-ipset-test}"
	table_tmp="${2:-ipset-test_tmp}"
	table_list=($(${BIN_IPSET} list -n))

	if [[ " ${table_list[*]} " =~ " ${table} " ]] && [[ " ${table_list[*]} " =~ " ${table_tmp} " ]]; then
		${BIN_IPSET} -W "${table_tmp}" "${table}"
		${BIN_IPSET} destroy "${table_tmp}"
	else
		__LOG "err" "ERROR($FUNCNAME): some tables do not exist." 1>&2
		return 1
	fi

}


__FIREWALL(){

	local ip_list

	ip_list=$(curl -s --connect-timeout 5 https://wmp.wildix.com/firewall/ips | grep -Eo "([0-9]{1,3}[\.]){3}([0-9]{1,3}|[0-9]{1,3}/[0-9]{1,2})")

	__IPSET_ALLOC "${IPSET_TABLE}"     "hash:net"
	__IPSET_ALLOC "${IPSET_TABLE}_TMP" "hash:net"
	__CHECK_IPTABLES_RULE

	while read IP; do
		[ "$IP" ] && ${BIN_IPSET} add "${IPSET_TABLE}_TMP" "$IP"
	done <<< ${ip_list}

	__IPSET_SWAP "${IPSET_TABLE}" "${IPSET_TABLE}_TMP"

}


__IS_AMAZON_ALIVE() {

	if [ -z "$(${BIN_IP} r | grep -o "default")" ]; then
		timeout 10 dhclient eth0
		__LOG "err" "ERROR($FUNCNAME): network settings is empty (trying to recovery)" 1>&2
	fi

	[ "$(curl --connect-timeout 10 -s http://${AMAZON_API}/latest/meta-data/mac/)" ] && echo "1" || echo "0"

}


__WRITE_NET_CONF() {

	## delete anyway this rule
	rm -f /etc/systemd/network/*.link

	case "${1}" in

		add|ADD)

			## write config in file
			cat << 'END' > /etc/network/interfaces
# The loopback network interface
auto lo
iface lo inet loopback

# The primary network interface
iface eth0 inet manual
iface eth0 inet6 manual

auto eth1
allow-hotplug eth1
iface eth1 inet dhcp
iface eth1 inet6 manual
END

			echo -e "[Match]\nMACAddress=${2}\n\n[Link]\nName=eth1" > ${3}
			__LOG "notice" "Network configuration written in systemd ---> reboot..."
			sync && reboot && exit 0
		;;

		*)

			## write config in file
			cat << 'END' > /etc/network/interfaces
# The loopback network interface
auto lo
iface lo inet loopback

# The primary network interface
auto eth0
allow-hotplug eth0
iface eth0 inet dhcp
iface eth0 inet6 manual
END

			__LOG "notice" "Default network configuration written in systemd ---> reboot..."
			sync && reboot && exit 0
		;;

	esac

}


__SET_SERIAL(){

	local link_eth_01 mac_systemd mac_amazon mac_meta

	link_eth_01='/etc/systemd/network/01-eth1.link'

	[ -f '/etc/udev/rules.d/70-persistent-net.rules' ] && rm -f /etc/udev/rules.d/70-persistent-net.rules
	[ -f "${link_eth_01}" ] && mac_systemd="$(awk -F '=' '$1 ~ "MACAddress" {print $2}' ${link_eth_01})"

	if [ "$(__IS_AMAZON_ALIVE)" == "1" ] ; then
		mac_amazon="$(curl --connect-timeout 10 -s http://${AMAZON_API}/latest/meta-data/mac/)"
		mac_meta="$(curl --connect-timeout 10 -s http://${AMAZON_API}/latest/meta-data/tags/instance/pbxSerial)"
	fi

	__LOG "notice" "mac_meta: ${mac_meta}\nmac_amazon: ${mac_amazon}\nmac_systemd: ${mac_systemd}" "-e"

	## add rule for rename eth0 to eth1
	if [[ "${mac_amazon}" =~ ^[0-9a-zA-Z]{1,2}\:[0-9a-zA-Z]{1,2}\:[0-9a-zA-Z]{1,2}\:[0-9a-zA-Z]{1,2}\:[0-9a-zA-Z]{1,2}\:[0-9a-zA-Z]{1,2}$ ]]; then
		if [ "${mac_amazon}" != "${mac_systemd}" ]; then
			__WRITE_NET_CONF "add" "${mac_amazon}" "${link_eth_01}"
		else
			__LOG "notice" "Network configuration correct ---> continues"
		fi
	else
		__WRITE_NET_CONF
	fi

	## add fake interface
	##/sbin/modprobe dummy
	${BIN_IP} link add dummy0 type dummy

	if [ "$(${BIN_IP} a | grep -wo dummy0)" ]; then
		[ "$mac_meta" ] && ${BIN_IFCONFIG} dummy0 hw ether "$mac_meta"
		${BIN_IP} l s name eth0 dev dummy0
		${BIN_IP} l s eth0 up
	fi

}


__SET_SSH_KEY(){

	local ssh_key key_file

	bin_aws="$(which aws)"

	if [ -z "${bin_aws}" ]; then
		__LOG "err" "ERROR($FUNCNAME): some binaries don't exist. exit."
		return 10
	fi

	ssh_key="$(${bin_aws} secretsmanager get-secret-value --region eu-central-1 --secret-id pbx_ssh_key --query SecretString --output text | cut -d: -f2 | tr -d \"})"
	key_file='/etc/skel/.ssh/authorized_keys'

	## main check
	if [ ! "${ssh_key}" ]; then
		__LOG "err" "ERROR($FUNCNAME): can't get ssh key ---> continues"
		return 1
	elif [ "$(echo "${ssh_key}" > /tmp/ssh-key.pub; \
		ssh-keygen -lf /tmp/ssh-key.pub &> /dev/null; \
		echo "$?")" -ne 0 ]; then
		__LOG "err" "ERROR($FUNCNAME): key file [${key_file}] is currupted."
		return 1
	fi


	## check ssh-key
	if [ -f "${key_file}" ]; then
		if [ ! "$(grep "${ssh_key}" /etc/skel/.ssh/authorized_keys)" ]; then
			mkdir -p /etc/skel/.ssh && sed -i '/pbxs/d' ${key_file} && echo "${ssh_key}" >> ${key_file}
			__LOG "notice" "ssh: adding key ---> write in ${key_file}"
		else
			__LOG "notice" "ssh: key already added ---> continues"
		fi
	else
		mkdir -p /etc/skel/.ssh && echo "${ssh_key}" >> ${key_file}
		__LOG "notice" "ssh: creates a new key ---> write in ${key_file}"
	fi

	## syncing files to admin directory
	mkdir -p /home/admin && rsync -ar /etc/skel/ /home/admin/ && chown -R admin:admin /home/admin

}


__RESIZE_PARTITION(){

	local bin_growpart bin_resize2fs bin_lsblk disk_row_data blk_disk_size resize_min_size resize_mountpoint \
		blk_name blk_maj_min blk_size blk_type blk_mountpoint blk_pkname

	resize_mountpoint="${1:-/}"
	resize_min_size="1G"
	bin_growpart="$(which growpart)"
	bin_resize2fs="$(which resize2fs)"
	bin_lsblk="$(which lsblk)"

	if [ -z "${bin_growpart}" ] || [ -z "${bin_resize2fs}" ] || [ -z "${bin_lsblk}" ]; then
		__LOG "err" "ERROR($FUNCNAME): some binaries don't exist. exit."
		return 10
	fi

	disk_row_data="$(${bin_lsblk} -lpnb -o NAME,MAJ:MIN,SIZE,TYPE,MOUNTPOINT,PKNAME)"

	while read blk_name blk_maj_min blk_size blk_type blk_mountpoint blk_pkname; do

		if [ "${blk_mountpoint}" == "${resize_mountpoint}" ]; then
			if [ ${blk_type} == 'part' ]; then

				blk_disk_size="$(awk '$1 == "'"${blk_pkname}"'" {print $3}' <<< ${disk_row_data})"

				## calculate if the disk size is larger than the partition in it
				if [ "$((${blk_disk_size} - ${blk_size}))" -ge "$(numfmt --from=iec ${resize_min_size})" ]; then

					__LOG "info" "INFO($FUNCNAME): root partition [${blk_name}][$(numfmt --to=iec ${blk_size})] is smaller than disk [${blk_pkname}][$(numfmt --to=iec ${blk_disk_size})], resizing."
					${bin_growpart} ${blk_pkname} ${blk_maj_min#*:} 2>&1 | __LOG "debug"
					${bin_resize2fs} ${blk_name} 2>&1 | __LOG "debug"

				fi
			else
				__LOG "err" "ERROR($FUNCNAME): the root partition [${blk_mountpoint}] is another type [${blk_type}]."
			fi
		fi

	done <<< ${disk_row_data}

}


__RESET_PBX(){

	local set_net pbx_serial

	set_net='0'

	case "$(awk -F ': ' '/model name/{print $2}' /proc/cpuinfo)" in
		*D2550*) set_net='1' ;;
		*J1800*) set_net='1' ;;
		*J1900*) set_net='1' ;;
	esac

	if [ "$set_net" -eq '1' ]; then

	cat << 'END' > /etc/network/interfaces
########################################################################
# INTERFACES FILE WAS GENERATED BY WMS. PLEASE DO NOT CHANGE IT MANUALY

auto lo
iface lo inet loopback

auto eth0
allow-hotplug eth0
iface eth0 inet dhcp

auto eth1
allow-hotplug eth1
iface eth1 inet static
	address 10.135.0.1
	netmask 255.255.0.0
	broadcast 10.135.255.255
	network 10.135.0.0
END

	fi

	# Wipe files
	rm -f /checkfs.log
	rm -f /etc/serial
	rm -f /root/.bash_history
	rm -f /rw2/etc/pbx/auth.conf
	rm -f /rw2/etc/pbx/geo_location.conf
	rm -f /rw2/etc/sf2
	rm -f /rw2/var/lib/wmsdb.d/wmsdb*
	rm -f /var/www/upgrade/wmsupgrade.out*
	rm -rf /etc/openvpn/*
	rm -rf /root/.nano
	rm -rf /root/.ssh
	echo "[Settings]" > /etc/wms.conf
	> /etc/dnsmasq.d/hosts-wms.conf
	> /rw2/etc/currentserial
	> /var/log/wms.log
	chown -f wms:wms /rw2/etc/currentserial /rw2/etc/sf2

	# Setting localtime to GMT0
	ln -sf /usr/share/zoneinfo/Europe/Rome /rw2/etc/localtime
	echo "Europe/Rome" > /rw2/etc/timezone

	pbx_serial="$(/usr/sbin/serial)"
	hostname "${pbx_serial}"
	echo "${pbx_serial}" > /etc/hostname

	[ -f '/etc/network/if-up.d/hosts' ] && /etc/network/if-up.d/hosts update
	[ -f '/usr/sbin/init_wmsdb' ] && /usr/sbin/init_wmsdb
	[ -f '/usr/sbin/ldapcmd' ] && /usr/sbin/ldapcmd reset
	${BIN_USERMOD} -p "$(mkpasswd wildix)" admin
	${BIN_SYSTEMCTL} disable pbx-reset.service
	echo -e "pbx-reset --> done" | logger -t pbx-reset

}


__INVERT_NETWORK(){

	local mac

	[ -f '/etc/udev/rules.d/70-persistent-net.rules' ] && rm -f /etc/udev/rules.d/70-persistent-net.rules

	for INTERFACE in $(ls /sys/class/net/ | awk '$1 ~ "^eth[0-9]$" {print $1}'); do
		mac="$(cat /sys/class/net/$INTERFACE/address)"

		case "$INTERFACE" in
			eth0) echo -e "[Match]\nMACAddress=${mac}\n\n[Link]\nName=eth3" > /etc/systemd/network/03-eth3.link ;;
			eth1) echo -e "[Match]\nMACAddress=${mac}\n\n[Link]\nName=eth2" > /etc/systemd/network/02-eth2.link ;;
			eth2) echo -e "[Match]\nMACAddress=${mac}\n\n[Link]\nName=eth1" > /etc/systemd/network/01-eth1.link ;;
			eth3) echo -e "[Match]\nMACAddress=${mac}\n\n[Link]\nName=eth0" > /etc/systemd/network/00-eth0.link ;;
		esac
	done

	${BIN_SYSTEMCTL} disable pbx-network-invert.service
	sync && reboot

}


__BEFORE_REBOOT(){

	local backup_log_dir bin_php time update_stage2_name update_stage2_log

	bin_php="$(which php)"
	time=$(date '+%d%m%Y_%H-%M')
	backup_log_dir='/rw2/var/log'
	update_stage2_name='upgrade_stage2'
	update_stage2_log="${backup_log_dir}/${update_stage2_name}_${time}.log"

	## Backup syslog to /rw2
	[ ! -d "${backup_log_dir}" ] && sudo mkdir -p ${backup_log_dir}

	## Run second stage upgrade before reboot
	for attempt in {1..10}; do

		if [ "$(pgrep -c ${update_stage2_name})" -eq '0' ]; then
			if [ "${bin_php}" ]; then
				${bin_php} /var/www/upgrade/${update_stage2_name}.php &>> ${update_stage2_log} || true
			else
				__LOG "err" "ERROR($FUNCNAME): can't find php, the update_stage2 is skip." &>> ${update_stage2_log}
			fi
			break
		else
			__LOG "err" "ERROR($FUNCNAME): the ${update_stage2_name} is already running, waiting 5 sec (${attempt})." &>> ${update_stage2_log}
			sleep 5
		fi

	done

	## cleanup old files
	find ${backup_log_dir} -type f -name '*.tar' -mmin +10080 -delete
	find ${backup_log_dir} -type f -name '*.log' -mmin +10080 -delete

	## move logs to backup dir
	cd /var/log && sudo tar cf logs_${time}.tar wms.*
	mv /var/log/logs_${time}.tar ${backup_log_dir}
	sudo chown -R wms:wms ${backup_log_dir}

}


case "$1" in
	reset)
		__RESET_PBX > /tmp/reset-pbx.log ## reset and cleanup a new one installed pbx
	;;
	serial_init)
		__SET_SERIAL            ## add eth0 with pbxSerial tag
	;;
	ssh_key)
		__SET_SSH_KEY           ## add ssh-key to /etc/skel/.ssh/authorized_keys
	;;
	resize_partitions)
		__RESIZE_PARTITION "/"  ## resize partition
	;;
	network_invert)
		__INVERT_NETWORK        ## invert network interface name
	;;
	firewall)
		__FIREWALL              ## wildix global firewall
	;;
	before_reboot)
		__BEFORE_REBOOT         ## execute logic "before reboot"
	;;
esac

exit 0
